package org.expeditee.network;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.ServerSocket;
import java.net.Socket;

import org.expeditee.gui.FrameIO;

public class ImageSaver extends DefaultServer {
	
	/**
	 * Reads an image storage request, and saves the image on the server
	 * 
	 * @author jts21
	 *
	 */
	private static class ImageReceiver extends Thread {
		
		private Socket client;
		
		public ImageReceiver(Socket client) {
			super("ImageReceiver");
			this.client = client;
		}
		
		@Override
		public void run() {
			try {
				this.client.shutdownOutput();
				InputStream is = this.client.getInputStream();
				// file name length is the first 2 bytes
	            int fileNameLen = (is.read() << 8) | is.read();
	            // file length is the next 4 bytes
	            int fileLen = (is.read() << 24) | (is.read() << 16) | (is.read() << 8) | is.read();
	            // file name is the remaining bytes of the header, encoded in UTF-8
	            byte[] fileName = new byte[fileNameLen];
	            is.read(fileName);
	            File f = new File(FrameIO.IMAGES_PATH + new String(fileName, ImageServer.CHARSET));
	            f.mkdirs();
	            // file is the remaining bytes of the stream
	            byte[] fileData = new byte[fileLen];
	            is.read(fileData);
	            // write the file
	            FileOutputStream out = new FileOutputStream(f);
	            out.write(fileData);
	            out.close();
	            this.client.close();
            } catch (IOException e) {
	            e.printStackTrace();
            }
		}
	}
	
	public final static int OFFSET = 6;
	
	private ServerSocket socket;

	public ImageSaver(int port) throws IOException {
		super("ImageSaver");
		this.socket = new ServerSocket(port + OFFSET);
	}

	@Override
	protected void listenForMessages() throws IOException {
		Socket client = socket.accept();
		// should probably only allow a limited number of threads here
		new ImageReceiver(client).start();
	}
	
	@Override
	public void run() {
		//MessageBay.displayMessage(this.getName() + " started on port "
		//		+ socket.getLocalPort());

		while (!_stop) {
			try {
				listenForMessages();
	
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		try {
	        socket.close();
        } catch (IOException e) {
	        e.printStackTrace();
        }
	}

}
