package org.expeditee.network;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.ServerSocket;
import java.net.Socket;
import java.nio.charset.Charset;

import org.expeditee.gui.FrameIO;

public class ImageServer extends DefaultServer {
	
	/**
	 * Reads an image request, and sends the image back if it was found
	 * 
	 * @author jts21
	 *
	 */
	private static class ImageSender extends Thread {
		
		private Socket client;
		
		public ImageSender(Socket client) {
			super("ImageSender");
			this.client = client;
		}
		
		@Override
		public void run() {
			try {
				BufferedOutputStream os = new BufferedOutputStream(this.client.getOutputStream());
				InputStream is = this.client.getInputStream();
				// file name length is the first 2 bytes
	            int fileNameLen = (is.read() << 8) | is.read();
	            // file name is the remaining bytes of the stream, encoded in UTF-8
	            byte[] fileName = new byte[fileNameLen];
	            is.read(fileName);
	            this.client.shutdownInput();
	            File f = new File(FrameIO.IMAGES_PATH + new String(fileName, ImageServer.CHARSET));
	            if(f.exists()) {
	            	// send the file over the socket
	            	int fileLen = (int) f.length();
	            	System.out.println("Sending " + new String(fileName, ImageServer.CHARSET) + " (" + fileLen + " bytes)");
	            	byte[] fileLenBytes = new byte[4];
	            	fileLenBytes[0] = (byte) ((fileLen >> 24) & 0xFF);
	            	fileLenBytes[1] = (byte) ((fileLen >> 16) & 0xFF);
	            	fileLenBytes[2] = (byte) ((fileLen >> 8) & 0xFF);
	            	fileLenBytes[3] = (byte) ((fileLen) & 0xFF);
	            	os.write(fileLenBytes);
	            	FileInputStream in = new FileInputStream(f);
	            	byte[] buf = null;
	            	if(in.available() > 1024) {
	            		buf = new byte[1024];
	            		while(in.available() > 1024) {
	            			in.read(buf);
	            			os.write(buf);
	            		}
	            	}
	            	buf = new byte[in.available()];
	            	in.read(buf);
        			os.write(buf);
        			os.flush();
        			in.close();
	            }
            	this.client.close();
            } catch (IOException e) {
	            e.printStackTrace();
            }
		}
	}
	
	public final static Charset CHARSET = Charset.forName("UTF-8");
	
	public final static int OFFSET = 5;
	
	private ServerSocket socket;

	public ImageServer(int port) throws IOException {
		super("ImageServer");
		this.socket = new ServerSocket(port + OFFSET);
	}

	@Override
	protected void listenForMessages() throws IOException {
		Socket client = socket.accept();
		// should probably only allow a limited number of threads here
		new ImageSender(client).start();
	}
	
	@Override
	public void run() {
		//MessageBay.displayMessage(this.getName() + " started on port "
		//		+ socket.getLocalPort());

		while (!_stop) {
			try {
				listenForMessages();
	
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		try {
	        socket.close();
        } catch (IOException e) {
	        e.printStackTrace();
        }
	}

}
